/*
 * exempi - xmp.h
 *
 * Copyright (C) 2007 Hubert Figuiere
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA
 */


#ifndef __EXEMPI_XMP_H_
#define __EXEMPI_XMP_H_

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

/** pointer to XMP packet. Opaque. */
typedef struct _Xmp *XmpPtr;
typedef struct _XmpTree *XmpTreePtr;
typedef struct _XmpSchema *XmpSchemaPtr;
typedef struct _XmpValue *XmpValuePtr;
typedef struct _XmpSchemaIterator *XmpSchemaIterator;
typedef struct _XmpPropertyIterator *XmpPropertyIterator;

/** Create a new XMP packet
 * @param buffer the buffer to load data from. UTF-8 encoded.
 * @param len the buffer length in byte
 * @return the packet point. Must be free with xmp_free()
 */
XmpPtr xmp_new(const char *buffer, size_t len);

/** Get an XMP property from the XMP packet
 * @param xmp the XMP packet
 * @param schema
 * @param name
 * @param value
 * @param len
 * @return and error code.
 */
int xmp_get_property(XmpPtr xmp, const char *schema, 
										 const char *name, 
										 char *value, size_t len);

/** Set an XMP property from the XMP packet
 * @param xmp the XMP packet
 * @param schema
 * @param name
 * @param value 0 terminated string
 * @return and error code.
 */
int xmp_set_property(XmpPtr xmp, const char *schema, 
										 const char *name, const char *value);

/** Free the xmp packet
 * @param xmp the xmp packet to free
 */
void xmp_free(XmpPtr xmp);


/** Return an xmp tree
 * @param xmp the xmp packet
 * @return a pointer to the xmp tree
 * The returned point belong to the xmp packet.
 */
XmpTreePtr xmp_get_tree(XmpPtr xmp);

/** Get the iterator to walk through schemas
 * @param tree the XMP tree to walk in
 * @return an iterator positionned to the first item.
 * The iterator must be freed by the caller using 
 * xmp_schema_iter_free()
 */
XmpSchemaIterator xmp_tree_get_schema_iter(XmpTreePtr tree);
/** Move to the next element
 * @return 1 if still valid, 0 if reached the end.
 */
int xmp_schema_iter_next(XmpSchemaIterator iter);
/** Test if the schema iterator is valid 
 */
int xmp_schema_iter_is_valid(XmpSchemaIterator iter);
/** Get the schema name at the current iterator position
 * @return the name of the schema. The pointer belongs 
 * to the schema and can be invalidated if the XMP data
 * change.
 */
const char* xmp_schema_iter_name(XmpSchemaIterator iter);
/** Get the schema out of the iterator position
 * @return the schema at the iterator position. The pointer
 * belongs to the tree and can be invalidated if the XMP data
 * change.
 */
XmpSchemaPtr xmp_schema_iter_value(XmpSchemaIterator iter);
/** Free a schema iterator.
 * @param iter the iterator to free
 */
void xmp_schema_iter_free(XmpSchemaIterator iter);


/** Get the iterator to go through properties
 * @param schema the schema to walk through
 * @return the allocated iterator, positionned to the first item. 
 * The iteraotor must be freed by xmp_property_iter_free()
 */
XmpPropertyIterator xmp_schema_get_property_iter(XmpSchemaPtr schema);
/** Move the iterator to the next element
 * @return 1 if the iterator is still valid. 0 if the end
 * is reached.
 */
int xmp_property_iter_next(XmpPropertyIterator iter);
/** Test if the iterator is valid.
 */
int xmp_property_iter_is_valid(XmpPropertyIterator iter);
/** Get the property name at the iterator location 
 */
const char* xmp_property_iter_name(XmpPropertyIterator iter);
/** Get the property value at the iterator location 
 */
XmpValuePtr xmp_property_iter_value(XmpPropertyIterator iter);
/** Free the iterator 
 */
void xmp_property_iter_free(XmpPropertyIterator iter);


/** Get the string value. If it is not a string, return NULL
 * @param value the value to retrieve the string from
 * @return a zero terminated C string, or NULL. String belong
 * to the XMP value
 */
const char* xmp_value_get_string(XmpValuePtr value);

#ifdef __cplusplus
}
#endif

#endif
