/*
 * exempi - iterwrap.h
 *
 * Copyright (C) 2007 Hubert Figuiere
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA
 */



#ifndef __ITER_WRAP_H_
#define __ITER_WRAP_H_


namespace Capi {

	/** @brief a STL iterator wrapper
	 * @param C the container type
	 * The container should provide the value_type type
	 * a begin() and a end() method that match the STL 
	 * semantics. 
	 */
	template <class C>
	class IterWrap
	{
	public:
		IterWrap(C & container);
		
		bool next(); 
		bool isValid();
		typename C::value_type & data();
	private:
		C & m_container;
		typename C::iterator m_current;
	};
	
	

	/** build an IterWrap 
	 * @param container the container to iterate through
	 * The iterator is set at the beginning.
	 */
	template <class C>
	IterWrap<C>::IterWrap(C & container)
		: m_container(container)
	{
		m_current = container.begin();
	}
	
	/** Move to the next value.
	 * @return true if the iterator is NOT at the end.
	 */
	template <class C>
	bool IterWrap<C>::next()
	{
		++m_current;
		return !(m_current == m_container.end());
	}

	/** Tell if the iterator is valid.
	 * @return the validity of the iterator.
	 */
	template <class C>
	bool IterWrap<C>::isValid()
	{
		return (m_current != m_container.end());
	}
	
	/** return the iterator value
	 * @return the value pointed to by the iterator.
	 */
	template <class C>
	typename C::value_type & IterWrap<C>::data()
	{
		return *m_current;
	}

}



#endif
